==> 1.0.1.tar.gz: lib/src/components/util/route_transitions/expand_up_transition.dart <==
/*---------------------------------------------------------------------------------------------
*  Copyright (c) nt4f04und. All rights reserved.
*  Licensed under the BSD-style license. See LICENSE in the project root for license information.
*--------------------------------------------------------------------------------------------*/

import 'route_transitions.dart';
import 'package:flutter/material.dart';

// Used by all of the transition animations.
const Curve _transitionCurve = Cubic(0.20, 0.00, 0.00, 1.00);

// The new page slides upwards just a little as its clip
// rectangle exposes the page from bottom to top.
final Tween<Offset> _primaryTranslationTween = Tween<Offset>(
  begin: const Offset(0.0, 0.05),
  end: Offset.zero,
);

// The old page slides upwards a little as the new page appears.
final Tween<Offset> _secondaryTranslationTween = Tween<Offset>(
  begin: Offset.zero,
  end: const Offset(0.0, -0.025),
);

/// Creates customizable expand up route transition
///
/// By default acts pretty same as [OpenUpwardsPageTransitionsBuilder] - creates upwards expand in transition
class ExpandUpRouteTransition<T extends Widget, RouteT extends dynamic>
    extends RouteTransition<T, RouteT> {
  @override
  final T route;
  @override
  final RouteT routeType;
  @override
  BoolFunction checkEntAnimationEnabled;
  @override
  BoolFunction checkExitAnimationEnabled;
  @override
  final Curve entCurve;
  @override
  final Curve entReverseCurve;
  @override
  final Curve exitCurve;
  @override
  final Curve exitReverseCurve;
  @override
  final bool entIgnoreEventsForward;
  @override
  final bool exitIgnoreEventsForward;
  @override
  final bool exitIgnoreEventsReverse;
  @override
  UIFunction checkSystemUi;

  /// Begin offset for exit animation
  ///
  /// Defaults to [Offset.zero]
  final Offset exitBegin;

  /// End offset for exit animation
  ///
  /// Defaults to [const Offset(-0.3, 0.0)]
  final Offset exitEnd;

  /// If true, default material exit animation will be played
  final bool playMaterialExit;

  ExpandUpRouteTransition({
    @required this.route,
    this.routeType,
    this.checkEntAnimationEnabled = defBoolFunc,
    this.checkExitAnimationEnabled = defBoolFunc,
    this.entCurve = Curves.linearToEaseOut,
    this.entReverseCurve = Curves.easeInToLinear,
    this.exitCurve = Curves.linearToEaseOut,
    this.exitReverseCurve = Curves.easeInToLinear,
    this.entIgnoreEventsForward = false,
    this.exitIgnoreEventsForward = false,
    this.exitIgnoreEventsReverse = false,
    this.checkSystemUi,
    this.exitBegin = Offset.zero,
    this.exitEnd = const Offset(-0.2, 0.0),
    Duration transitionDuration = kNFRouteTransitionDuration,
    RouteSettings settings,
    bool opaque = true,
    bool maintainState = false,
    this.playMaterialExit = false,
  }) : super(
          route: route,
          transitionDuration: transitionDuration,
          settings: settings,
          opaque: opaque,
          maintainState: maintainState,
        ) {
    transitionsBuilder = (
      BuildContext context,
      Animation<double> animation,
      Animation<double> secondaryAnimation,
      Widget child,
    ) {
      final bool exitEnabled = checkExitAnimationEnabled();

      final bool ignore =
          entIgnoreEventsForward && animation.status == AnimationStatus.forward;
      final bool secondaryIgnore = exitIgnoreEventsForward &&
              secondaryAnimation.status == AnimationStatus.forward ||
          exitIgnoreEventsReverse &&
              secondaryAnimation.status == AnimationStatus.reverse;

      /// Wrap child for to use with material routes (difference from default child is that is has animation status completed check, that brakes theme ui switch)
      final Container materialWrappedChild = Container(
        color: Colors.black,
        child: FadeTransition(
          opacity: secondaryAnimation.status == AnimationStatus.forward
              // Dim route on exit
              ? exitDimTween.animate(
                  secondaryAnimation,
                )
              // Dim route on exit reverse, but less a little bit than on forward
              : secondaryAnimation.status == AnimationStatus.reverse
                  ? exitRevDimTween.animate(
                      secondaryAnimation,
                    )
                  // Do not dim in other cases
                  : constTween.animate(secondaryAnimation),
          child: IgnorePointer(
            // Disable any touch events on enter while in transition
            ignoring: ignore,
            child: IgnorePointer(
              // Disable any touch events on exit while in transition
              ignoring: secondaryIgnore,
              child: child,
            ),
          ),
        ),
      );

      return LayoutBuilder(
        builder: (BuildContext context, BoxConstraints constraints) {
          final Size size = constraints.biggest;

          final CurvedAnimation primaryAnimation = CurvedAnimation(
            parent: animation,
            curve: _transitionCurve,
            reverseCurve: _transitionCurve.flipped,
          );

          // Gradually expose the new page from bottom to top.
          final Animation<double> clipAnimation = Tween<double>(
            begin: 0.0,
            end: size.height,
          ).animate(primaryAnimation);

          final Animation<Offset> primaryTranslationAnimation =
              _primaryTranslationTween.animate(primaryAnimation);

          final Animation<Offset> secondaryTranslationAnimation =
              _secondaryTranslationTween.animate(
            CurvedAnimation(
              parent: secondaryAnimation,
              curve: _transitionCurve,
              reverseCurve: _transitionCurve.flipped,
            ),
          );

          return AnimatedBuilder(
            animation: animation,
            builder: (BuildContext context, Widget child) {
              return Container(
                alignment: Alignment.bottomLeft,
                child: ClipRect(
                  child: SizedBox(
                    height: clipAnimation.value,
                    child: OverflowBox(
                      alignment: Alignment.bottomLeft,
                      maxHeight: size.height,
                      child: child,
                    ),
                  ),
                ),
              );
            },
            child: playMaterialExit
                ? AnimatedBuilder(
                    animation: secondaryAnimation,
                    child: FractionalTranslation(
                      translation: primaryTranslationAnimation.value,
                      child: child,
                    ),
                    builder: (BuildContext context, Widget child) {
                      return FractionalTranslation(
                        translation: secondaryTranslationAnimation.value,
                        child: materialWrappedChild,
                      );
                    },
                  )
                : TurnableSlideTransition(
                    enabled: exitEnabled,
                    position: Tween(begin: exitBegin, end: exitEnd).animate(
                        CurvedAnimation(
                            parent: secondaryAnimation,
                            curve: exitCurve,
                            reverseCurve: exitReverseCurve)),
                    child: materialWrappedChild,
                  ),
          );
        },
      );
    };
  }
}

==> 1.0.2.tar.gz: lib/src/components/util/route_transitions/expand_up_transition.dart <==
/*---------------------------------------------------------------------------------------------
*  Copyright (c) nt4f04und. All rights reserved.
*  Licensed under the BSD-style license. See LICENSE in the project root for license information.
*--------------------------------------------------------------------------------------------*/

import 'route_transitions.dart';
import 'package:flutter/material.dart';

// Used by all of the transition animations.
const Curve _transitionCurve = Cubic(0.20, 0.00, 0.00, 1.00);

// The new page slides upwards just a little as its clip
// rectangle exposes the page from bottom to top.
final Tween<Offset> _primaryTranslationTween = Tween<Offset>(
  begin: const Offset(0.0, 0.05),
  end: Offset.zero,
);

// The old page slides upwards a little as the new page appears.
final Tween<Offset> _secondaryTranslationTween = Tween<Offset>(
  begin: Offset.zero,
  end: const Offset(0.0, -0.025),
);

/// Creates customizable expand up route transition
///
/// By default acts pretty same as [OpenUpwardsPageTransitionsBuilder] - creates upwards expand in transition
class ExpandUpRouteTransition<T extends Widget> extends RouteTransition<T> {
  @override
  final T route;
  @override
  BoolFunction checkEntAnimationEnabled;
  @override
  BoolFunction checkExitAnimationEnabled;
  @override
  final Curve entCurve;
  @override
  final Curve entReverseCurve;
  @override
  final Curve exitCurve;
  @override
  final Curve exitReverseCurve;
  @override
  final bool entIgnore;
  @override
  final bool exitIgnore;
  @override
  UIFunction checkSystemUi;

  /// Begin offset for exit animation
  ///
  /// Defaults to [Offset.zero]
  final Offset exitBegin;

  /// End offset for exit animation
  ///
  /// Defaults to [const Offset(-0.3, 0.0)]
  final Offset exitEnd;

  /// If true, default material exit animation will be played
  final bool playMaterialExit;

  ExpandUpRouteTransition({
    @required this.route,
    this.checkEntAnimationEnabled = defRouteTransitionBoolFunc,
    this.checkExitAnimationEnabled = defRouteTransitionBoolFunc,
    this.entCurve = Curves.linearToEaseOut,
    this.entReverseCurve = Curves.easeInToLinear,
    this.exitCurve = Curves.linearToEaseOut,
    this.exitReverseCurve = Curves.easeInToLinear,
    this.entIgnore = false,
    this.exitIgnore = false,
    this.checkSystemUi,
    this.exitBegin = Offset.zero,
    this.exitEnd = const Offset(-0.2, 0.0),
    Duration transitionDuration = kNFRouteTransitionDuration,
    Duration reverseTransitionDuration = kNFRouteTransitionDuration,
    RouteSettings settings,
    bool opaque = true,
    bool maintainState = false,
    this.playMaterialExit = false,
  }) : super(
          route: route,
          transitionDuration: transitionDuration,
          reverseTransitionDuration: reverseTransitionDuration,
          settings: settings,
          opaque: opaque,
          maintainState: maintainState,
        ) {
    transitionsBuilder = (
      BuildContext context,
      Animation<double> animation,
      Animation<double> secondaryAnimation,
      Widget child,
    ) {
      /// Wrap child for to use with material routes (difference from default child is that is has animation status completed check, that brakes theme ui switch)
      final Container materialWrappedChild = Container(
        color: Colors.black,
        child: FadeTransition(
          opacity: secondaryAnimation.status == AnimationStatus.forward
              // Dim route on exit
              ? exitDimTween.animate(
                  secondaryAnimation,
                )
              // Dim route on exit reverse, but less a little bit than on forward
              : secondaryAnimation.status == AnimationStatus.reverse
                  ? exitRevDimTween.animate(
                      secondaryAnimation,
                    )
                  // Do not dim in other cases
                  : constTween.animate(secondaryAnimation),
          child: IgnorePointer(
            // Disable any touch events on enter while in transition
            ignoring: ignore,
            child: IgnorePointer(
              // Disable any touch events on exit while in transition
              ignoring: secondaryIgnore,
              child: child,
            ),
          ),
        ),
      );

      return LayoutBuilder(
        builder: (BuildContext context, BoxConstraints constraints) {
          final Size size = constraints.biggest;

          final CurvedAnimation primaryAnimation = CurvedAnimation(
            parent: animation,
            curve: _transitionCurve,
            reverseCurve: _transitionCurve.flipped,
          );

          // Gradually expose the new page from bottom to top.
          final Animation<double> clipAnimation = Tween<double>(
            begin: 0.0,
            end: size.height,
          ).animate(primaryAnimation);

          final Animation<Offset> primaryTranslationAnimation =
              _primaryTranslationTween.animate(primaryAnimation);

          final Animation<Offset> secondaryTranslationAnimation =
              _secondaryTranslationTween.animate(
            CurvedAnimation(
              parent: secondaryAnimation,
              curve: _transitionCurve,
              reverseCurve: _transitionCurve.flipped,
            ),
          );

          return AnimatedBuilder(
            animation: animation,
            builder: (BuildContext context, Widget child) {
              return Container(
                alignment: Alignment.bottomLeft,
                child: ClipRect(
                  child: SizedBox(
                    height: clipAnimation.value,
                    child: OverflowBox(
                      alignment: Alignment.bottomLeft,
                      maxHeight: size.height,
                      child: child,
                    ),
                  ),
                ),
              );
            },
            child: playMaterialExit
                ? AnimatedBuilder(
                    animation: secondaryAnimation,
                    child: FractionalTranslation(
                      translation: primaryTranslationAnimation.value,
                      child: child,
                    ),
                    builder: (BuildContext context, Widget child) {
                      return FractionalTranslation(
                        translation: secondaryTranslationAnimation.value,
                        child: materialWrappedChild,
                      );
                    },
                  )
                : TurnableSlideTransition(
                    enabled: exitAnimationEnabled,
                    position: Tween(begin: exitBegin, end: exitEnd).animate(
                      CurvedAnimation(
                        parent: secondaryAnimation,
                        curve: exitCurve,
                        reverseCurve: exitReverseCurve,
                      ),
                    ),
                    child: materialWrappedChild,
                  ),
          );
        },
      );
    };
  }
}
